# frozen_string_literal: true

module Nichugi
  # The arbiter abstraction.
  class Arbiter
    attr_reader :position

    SQUARE_VALUES = [
      0,  1,  2,  3,  4,  3,  2,  1,  0,
      1,  5,  6,  7,  8,  7,  6,  5,  1,
      2,  6,  9, 10, 11, 10,  9,  6,  2,
      3,  7, 10, 12, 13, 12, 10,  7,  3,
      4,  8, 11, 13, 14, 13, 11,  8,  4,
      3,  7, 10, 12, 13, 12, 10,  7,  3,
      2,  6,  9, 10, 11, 10,  9,  6,  2,
      1,  5,  6,  7,  8,  7,  6,  5,  1,
      0,  1,  2,  3,  4,  3,  2,  1,  0
    ].freeze

    def initialize(position)
      @position = position
    end

    def north_better?
      return true if north_king_value > south_king_value
      return false if south_king_value > north_king_value

      return true if north_pieces_value > south_pieces_value
      return false if south_pieces_value > north_pieces_value

      return true if north_pieces_count > south_pieces_count
      return false if south_pieces_count > north_pieces_count

      true
    end

    private

    def north_pieces_square
      position.square.select { |_id, piece_name| piece_name.downcase == piece_name }
    end

    def south_pieces_square
      position.square.select { |_id, piece_name| piece_name.upcase == piece_name }
    end

    def north_king_square_id
      north_pieces_square.select { |_id, piece_name| piece_name.include?("-") }.keys.fetch(0)
    end

    def south_king_square_id
      south_pieces_square.select { |_id, piece_name| piece_name.include?("-") }.keys.fetch(0)
    end

    def north_king_value
      SQUARE_VALUES.fetch(north_king_square_id)
    end

    def south_king_value
      SQUARE_VALUES.fetch(south_king_square_id)
    end

    def north_pieces_value
      north_pieces_square.inject(0) do |sum, (id, _piece_name)|
        sum + SQUARE_VALUES.fetch(id)
      end
    end

    def south_pieces_value
      south_pieces_square.inject(0) do |sum, (id, _piece_name)|
        sum + SQUARE_VALUES.fetch(id)
      end
    end

    def north_pieces_count
      north_pieces_square.keys.length
    end

    def south_pieces_count
      south_pieces_square.keys.length
    end
  end
end
